unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls, MyOpenGL,
  Vcl.Imaging.jpeg;

type
  TForm1 = class(TForm)
    Panel_Top: TPanel;
    Panel_Draw: TPanel;
    Edit_FPS: TEdit;
    Label1: TLabel;
    Button_Draw: TButton;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button_DrawClick(Sender: TObject);
    procedure Edit_FPSChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Panel_DrawMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Panel_DrawMouseLeave(Sender: TObject);
    procedure Panel_DrawMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Panel_DrawMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  private
    // for creating multiple forms
    AnotherForms : array of TForm1;

    // for animation
    counter : integer;

    // mouse control
    MouseLastX : integer;
    MouseLastY : integer;
    MouseDown : boolean;

    procedure MyCallBack(const DC : HDC; const Canvas : TControl);
  public
    MyCanvas : TMyOpenGL;
  end;

var
  Form1 : TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  MyCanvas := TMyOpenGL.Create(Panel_Draw, MyCallBack); // create new OpenGL context

  // default setting
  if Assigned(MyCanvas) then
    with MyCanvas do
    begin
      SetRenderFPS(0); // set manualy driven rendering
      SetCameraPosition(0, 0, -1000); // set camera position
    end;

  // init
  counter := 0;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  i : integer;
begin
  for i := 0 to length(AnotherForms) - 1 do
    if Assigned(TForm1(AnotherForms[i])) then
      TForm1(AnotherForms[i]).Close;

  if Assigned(MyCanvas) then MyCanvas.Free; // free OpenGL context
end;

procedure TForm1.MyCallBack(const DC : HDC; const Canvas : TControl);
var
  Grid1 : TLineObject;
  Grid2 : TLineObject;
  Axis : TLineObject;
  Cube : TQuadsObject;
  Star : TPolygonObject;
begin
  inc(counter);

  // render callback
  with MyCanvas do
  begin
    RenderingBegin; // start

    // two grind as ceiling and floor
    CreateLineObject_Grid(20, 100, 1, lsSolid, clLime, 1.0, Grid1); // prepare model structure
    Grid1.Position := MakeVector3f(0, 120, 0); // transform model structure
    Grid1.Rotation.X := 90; // transform model structure
    DrawObject(Grid1); // draw model structure
    CreateLineObject_Grid(20, 100, 1, lsSolid, clGreen, 1.0, Grid2); // prepare model structure
    Grid2.Position := MakeVector3f(0, -120, 0); // transform model structure
    Grid2.Rotation.X := 90; // transform model structure
    DrawObject(Grid2); // draw model structure

    // axis cross at 0,0,0
    CreateLineObject_Axes(100, Axis); // prepare model structure
    DrawObject(Axis); // draw model structure

    // cube object
    Create3DObject_Cube(100, 100, 100, clRed, clGreen, ClYellow, clBlue, clSilver, clLime, 1.0, Cube); // prepare model structure
    Cube.Position := MakeVector3f(250, 0, 0); // transform model structure
    Cube.Rotation.Y := Cube.Rotation.Y - (counter - (1 * GetDeltaTime)); // animation
    DrawObject(Cube); // draw model structure

    // star object
    Create3DObject_Star(50, 110, 6, true, clYellow, clBlue, 1.0, Star); // prepare model structure
    Star.Position := MakeVector3f(-250, 0, 0); // transform model structure
    Star.Rotation.X := Star.Rotation.X - (counter - (1 * GetDeltaTime)); // animation
    DrawObject(Star); // draw model structure

    RenderingEnd; // end
  end;
end;

procedure TForm1.Panel_DrawMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseLastX := X;
  MouseLastY := Y;
  MouseDown := true;
end;

procedure TForm1.Panel_DrawMouseLeave(Sender: TObject);
begin
  MouseDown := false;
end;

procedure TForm1.Panel_DrawMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  // camera orbit around 0, 0, 0
  if MouseDown then
    with MyCanvas do
    begin
      OrbitUp(1000, (Y - MouseLastY) * GetDeltaTime);
      OrbitLeft(1000, (X - MouseLastX) * GetDeltaTime, false);
    end;

  MouseLastX := X;
  MouseLastY := Y;
end;

procedure TForm1.Panel_DrawMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseDown := false;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  SetLength(AnotherForms, length(AnotherForms) + 1);
  AnotherForms[length(AnotherForms) - 1] := TForm1.Create(nil);
  with AnotherForms[length(AnotherForms) - 1] do
  begin
    Button1.Visible := false;
    Show;
  end;
end;

procedure TForm1.Button_DrawClick(Sender: TObject);
begin
  MyCanvas.Render; // manual render
end;

procedure TForm1.Edit_FPSChange(Sender: TObject);
var
  intV : integer;
  code : integer;
begin
  // change FPS of MyOpenGL
  // 0 = manual
  // 1..100 = cyclic driven by inner timer
  val(Edit_FPS.Text, intV, code);
  if code = 0 then
  begin
         if intV < 0   then intV := 0
    else if intV > 100 then intV := 100;

    MyCanvas.SetRenderFPS(intV); // set FPS

    if intV in [1..100] then Button_Draw.Enabled := false
                        else Button_Draw.Enabled := true;
  end;
end;

end.
